package test.projet.ejb.session;

import javax.ejb.Stateless;
import javax.persistence.Query;

import test.projet.ejb.client.entity.Civilite;
import test.projet.ejb.client.enums.PersistenceModeEnum;
import test.projet.ejb.client.exception.CauseException;
import test.projet.ejb.client.exception.ValidationException;
import test.projet.ejb.client.session.CiviliteFacadeLocal;
import test.projet.ejb.client.utils.QueryUtils;
import test.projet.ejb.client.utils.Utils;

@Stateless
public class CiviliteFacade extends AbstractFacade<Civilite> implements CiviliteFacadeLocal
{

	@Override
	public void check(Civilite entity, PersistenceModeEnum persistenceMode) throws ValidationException
	{
		ValidationException ve = null;

		/*
		 * Exécution des contrôles standards
		 */
		try 
		{
			super.check(entity, persistenceMode);
			ve = new ValidationException();
		}
		catch (ValidationException e)
		{
			ve = e;
		}
		
				
		/*
		 * Test de l'unicité du code
		 */
		if (Utils.isSet(entity.getAbrege()))
		{
			StringBuilder sb = new StringBuilder();
			sb.append("select count(a) from Civilite a where upper(a.abrege) = upper(:abrege)");
			
			QueryUtils.addQueryParameter(sb, "a.uid", "uid", "and", "<>", entity.getUid());
			
			Query query = getEntityManager().createQuery(sb.toString());
			
			query.setParameter("abrege", entity.getAbrege());
			QueryUtils.setQueryParameter(query, "uid", entity.getUid());
			
			Number count = (Number)query.getSingleResult();
			if (count.intValue() > 0)
			{
				ve.addCauseException(new CauseException("exception.duplicate", new Object[]{"civilite.abrege", entity.getAbrege()}));
			}
		}

		/*
		 * Erreur(s) rencontrée(s) ?
		 */
		if (ve.hasErrors())
		{
			throw ve;
		}
	}

}
