package test.projet.ejb.session;

import java.util.HashSet;
import java.util.List;

import javax.ejb.EJB;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import test.projet.ejb.client.dto.DTOCivilite;
import test.projet.ejb.client.dto.DTOPersonne;
import test.projet.ejb.client.dto.DTOPersonneTelephone;
import test.projet.ejb.client.dto.DTOTelephoneType;
import test.projet.ejb.client.entity.Civilite;
import test.projet.ejb.client.entity.Personne;
import test.projet.ejb.client.entity.PersonneTelephone;
import test.projet.ejb.client.entity.TelephoneType;
import test.projet.ejb.client.session.CiviliteFacadeLocal;
import test.projet.ejb.client.session.MonProjetFacadeLocal;
import test.projet.ejb.client.session.PersonneFacadeLocal;
import test.projet.ejb.client.session.TelephoneTypeFacadeLocal;
import test.projet.ejb.client.utils.QueryResult;
import test.projet.ejb.client.utils.QueryUtils;

@Stateless
public class MonProjetFacade implements MonProjetFacadeLocal
{
	@PersistenceContext(name = "MonProjetEJBPU")
	private EntityManager entityManager;
	
	@EJB
	private PersonneFacadeLocal ejbPersonne;
	@EJB
	private TelephoneTypeFacadeLocal ejbTelephoneType;
	@EJB
	private CiviliteFacadeLocal ejbCivilite;
	
	/*
	 * Personnes
	 */

	@Override
	public DTOPersonne getPersonne(Integer uid)
	{
		Personne personne = ejbPersonne.get(uid);
		return personne2DTOPersonne(personne);
	}
	

	@Override
	public DTOPersonne saveOrUpdatePersonne(DTOPersonne dtoPersonne) throws Exception
	{
		Personne personne = dtoPersonne2Personne(dtoPersonne);
		
		personne = ejbPersonne.saveOrUpdate(personne);
				
		return personne2DTOPersonne(personne);
	}


	@Override
	public void deletePersonne(Integer uid) throws Exception
	{
		ejbPersonne.delete(uid);
	}


	/**
	 * Recherche de personnes
	 */
	@SuppressWarnings("unchecked")
	@Override
	public QueryResult<DTOPersonne> recherchePersonnes(String prenom, String nom, Character genre, boolean count, int first, int pageSize, String sortField, String sortOrder)
	{
		QueryResult<DTOPersonne> result = new QueryResult<>();
		StringBuilder sb = new StringBuilder();

		if (count)
		{
			StringBuilder sbCount = new StringBuilder(sb);
			sbCount.append("select count(a) from Personne a where 1=1");
			QueryUtils.addQueryParameter(sbCount, "a.genre", "genre", "and", "=", genre);
			QueryUtils.addQueryParameter(sbCount, "a.prenom", "prenom", "and", "like", prenom);
			QueryUtils.addQueryParameter(sbCount, "a.nom", "nom", "and", "like", nom);

			Query queryCount = entityManager.createQuery(sbCount.toString());

			QueryUtils.setQueryParameter(queryCount, "genre", genre);
			QueryUtils.setQueryParameter(queryCount, "prenom", prenom);
			QueryUtils.setQueryParameter(queryCount, "nom", nom);

			result.setTotalRecords((Long) queryCount.getSingleResult());
		}

		sb.append("select a from Personne a where 1=1");

		QueryUtils.addQueryParameter(sb, "a.genre", "genre", "and", "=", genre);
		QueryUtils.addQueryParameter(sb, "a.prenom", "prenom", "and", "like", prenom);
		QueryUtils.addQueryParameter(sb, "a.nom", "nom", "and", "like", nom);

		Query query = entityManager.createQuery(sb.toString());

		QueryUtils.setQueryParameter(query, "genre", genre);
		QueryUtils.setQueryParameter(query, "prenom", prenom);
		QueryUtils.setQueryParameter(query, "nom", nom);

		if (first > 0)
		{
			query.setFirstResult(first);
		}
		if (pageSize > 0)
		{
			query.setMaxResults(pageSize);
		}

		for (Personne personne : (List<Personne>) query.getResultList())
		{
			result.getRecords().add(personne2DTOPersonne(personne));
		}

		return result;
	}

	/**
	 * Convertion d'un objet Personne en DTOPersonne
	 * 
	 * @param personne
	 * @return
	 */
	public DTOPersonne personne2DTOPersonne(Personne personne)
	{
		if (personne == null)
		{
			return null;
		}
		
		DTOPersonne dtoPersonne = new DTOPersonne(personne.getUid(), 
				personne.getGenre(), 
				personne.getPrenom(), 
				personne.getNom(), 
				personne.getAdresse1(), 
				personne.getAdresse2(), 
				personne.getCodePostal(), 
				personne.getVille(), 
				civilite2DTOCivilite(personne.getCivilite()),
				personne.getMajDate(),
				new HashSet<DTOPersonneTelephone>());

		for (PersonneTelephone telephone : personne.getPersonneTelephones())
		{
			dtoPersonne.getTelephones().add(
					new DTOPersonneTelephone(telephone.getUid(), 
					telephoneType2DTOTelephoneType(telephone.getTelephoneType()), 
					telephone.getNumero()));
		}
		
		return dtoPersonne;
	}

	/**
	 * Convertion d'un objet DTOPersonne en Personne
	 * 
	 * @param personne
	 * @return
	 */
	public Personne dtoPersonne2Personne(DTOPersonne dtoPersonne)
	{
		if (dtoPersonne == null)
		{
			return null;
		}

		Personne personne = new Personne(dtoPersonne.getUid(), 
				dtoCivilite2Civilite(dtoPersonne.getCivilite()), 
				dtoPersonne.getGenre(), 
				dtoPersonne.getPrenom(), 
				dtoPersonne.getNom(), 
				dtoPersonne.getAdresse1(), 
				dtoPersonne.getAdresse2(), 
				dtoPersonne.getCodePostal(), 
				dtoPersonne.getVille(), 
				dtoPersonne.getMajDate(), 
				new HashSet<>());
		
		for (DTOPersonneTelephone personneTelephone : dtoPersonne.getTelephones())
		{
			personne.getPersonneTelephones().add(new PersonneTelephone(personneTelephone.getUid(),
					personne,
					dtoTelephoneType2TelephoneType(personneTelephone.getTelephoneType()),
					personneTelephone.getNumero()));
		}
		
		return personne;
	}

	
	/*
	 * TelephoneType
	 */

	
	@SuppressWarnings("unchecked")
	@Override
	public QueryResult<DTOTelephoneType> rechercheTelephoneTypes(boolean count, int first, int pageSize, String sortField, String sortOrder)
	{
		QueryResult<DTOTelephoneType> result = new QueryResult<>();
		StringBuilder sb = new StringBuilder();

		sb.append("select a from TelephoneType a where 1=1");

		Query query = entityManager.createQuery(sb.toString());

		if (first > 0)
		{
			query.setFirstResult(first);
		}
		if (pageSize > 0)
		{
			query.setMaxResults(pageSize);
		}

		for (TelephoneType telephoneType : (List<TelephoneType>) query.getResultList())
		{
			result.getRecords().add(telephoneType2DTOTelephoneType(telephoneType));
		}

		return result;
	}

	@Override
	public DTOTelephoneType getTelephoneType(Integer uid)
	{
		if (uid == null)
		{
			return null;
		}
		
		TelephoneType telephoneType = entityManager.find(TelephoneType.class, uid);
		return telephoneType2DTOTelephoneType(telephoneType);
	}
	
	@Override
	public DTOTelephoneType saveOrUpdateTelephoneType(DTOTelephoneType dtoTelephoneType) throws Exception
	{
		TelephoneType telephoneType = dtoTelephoneType2TelephoneType(dtoTelephoneType);
		telephoneType = ejbTelephoneType.saveOrUpdate(telephoneType);
		return telephoneType2DTOTelephoneType(telephoneType);
	}


	@Override
	public void deleteTelephoneType(Integer uid) throws Exception
	{
		ejbTelephoneType.delete(uid);
	}


	public DTOTelephoneType telephoneType2DTOTelephoneType(TelephoneType telephoneType)
	{
		if (telephoneType == null)
		{
			return null;
		}
		
		return new DTOTelephoneType(telephoneType.getUid(),
				telephoneType.getCode(),
				telephoneType.getLibelle());
	}


	public TelephoneType dtoTelephoneType2TelephoneType(DTOTelephoneType dtoTelephoneType)
	{
		if (dtoTelephoneType == null)
		{
			return null;
		}
		
		return new TelephoneType(dtoTelephoneType.getUid(),
				dtoTelephoneType.getCode(),
				dtoTelephoneType.getLibelle());
	}

	
	/*
	 * Civilité
	 */
	
	public DTOCivilite getCivilite(Integer uid)
	{
		if (uid == null)
		{
			return null;
		}
		Civilite civilite = entityManager.find(Civilite.class, uid);
		
		return civilite2DTOCivilite(civilite);
	}

	/**
	 * Recherche de civilités
	 */
	@SuppressWarnings("unchecked")
	@Override
	public QueryResult<DTOCivilite> rechercheCivilites(String abrege, String libelle, boolean count, int first, int pageSize, String sortField, String sortOrder)
	{
		QueryResult<DTOCivilite> result = new QueryResult<>();
		StringBuilder sb = new StringBuilder();

		if (count)
		{
			StringBuilder sbCount = new StringBuilder(sb);
			sbCount.append("select count(a) from Civilite a where 1=1");
			QueryUtils.addQueryParameter(sbCount, "a.abrege", "abrege", "and", "like", abrege);
			QueryUtils.addQueryParameter(sbCount, "a.libelle", "libelle", "and", "like", libelle);

			Query queryCount = entityManager.createQuery(sbCount.toString());

			QueryUtils.setQueryParameter(queryCount, "abrege", abrege);
			QueryUtils.setQueryParameter(queryCount, "libelle", libelle);

			result.setTotalRecords((Long) queryCount.getSingleResult());
		}

		sb.append("select a from Civilite a where 1=1");
		QueryUtils.addQueryParameter(sb, "a.abrege", "abrege", "and", "like", abrege);
		QueryUtils.addQueryParameter(sb, "a.libelle", "libelle", "and", "like", libelle);

		Query query = entityManager.createQuery(sb.toString());

		QueryUtils.setQueryParameter(query, "abrege", abrege);
		QueryUtils.setQueryParameter(query, "libelle", libelle);

		if (first > 0)
		{
			query.setFirstResult(first);
		}
		if (pageSize > 0)
		{
			query.setMaxResults(pageSize);
		}

		for (Civilite civilite : (List<Civilite>) query.getResultList())
		{
			result.getRecords().add(civilite2DTOCivilite(civilite));
		}

		return result;
	}
	
	@Override
	public void deleteCivilite(Integer uid)
	{
		ejbCivilite.delete(uid);
	}
	
	@Override
	public DTOCivilite saveOrUpdateCivilite(DTOCivilite dtoCivilite) throws Exception
	{
		Civilite civilite = dtoCivilite2Civilite(dtoCivilite);
		civilite = ejbCivilite.saveOrUpdate(civilite);
		return civilite2DTOCivilite(civilite);	
	}

	/**
	 * Convertion Entity -> DTO
	 * @param civilite
	 * @return
	 */
	public DTOCivilite civilite2DTOCivilite(Civilite civilite)
	{
		if (civilite == null)
		{
			return null;
		}
		
		return new DTOCivilite(civilite.getUid(), civilite.getAbrege(), civilite.getLibelle());
	}
	
	/**
	 * Convertion DTO -> Entity
	 * @param dtoCivilite
	 * @return
	 */
	public Civilite dtoCivilite2Civilite(DTOCivilite dtoCivilite)
	{
		if (dtoCivilite == null)
		{
			return null;
		}
		
		return new Civilite(dtoCivilite.getUid(), dtoCivilite.getAbrege(), dtoCivilite.getLibelle());
	}
	
}
