package test.projet.ejb.session;

import javax.ejb.Stateless;
import javax.persistence.Query;

import test.projet.ejb.client.entity.TelephoneType;
import test.projet.ejb.client.enums.PersistenceModeEnum;
import test.projet.ejb.client.exception.CauseException;
import test.projet.ejb.client.exception.ValidationException;
import test.projet.ejb.client.session.TelephoneTypeFacadeLocal;
import test.projet.ejb.client.utils.QueryUtils;
import test.projet.ejb.client.utils.Utils;

@Stateless
public class TelephoneTypeFacade extends AbstractFacade<TelephoneType> implements TelephoneTypeFacadeLocal
{

	@Override
	public void check(TelephoneType entity, PersistenceModeEnum persistenceMode) throws ValidationException
	{
		ValidationException ve = null;

		/*
		 * Exécution des contrôles standards
		 */
		try
		{
			super.check(entity, persistenceMode);
			ve = new ValidationException();
		}
		catch (ValidationException e)
		{
			ve = e;
		}		
		
		/*
		 * Test de l'unicité du code
		 */
		if (Utils.isSet(entity.getCode()))
		{
			StringBuilder sb = new StringBuilder();
			sb.append("select count(a) from TelephoneType a where upper(a.code) = upper(:code)");
			
			QueryUtils.addQueryParameter(sb, "a.uid", "uid", "and", "<>", entity.getUid());
			
			Query query = getEntityManager().createQuery(sb.toString());
			
			query.setParameter("code", entity.getCode());
			QueryUtils.setQueryParameter(query, "uid", entity.getUid());
			
			Number count = (Number)query.getSingleResult();
			if (count.intValue() > 0)
			{
				ve.addCauseException(new CauseException("exception.duplicate", new Object[]{"telephoneType.code", entity.getCode()}));
			}			
		}	
		
		/*
		 * Erreur(s) rencontrée(s) ?
		 */
		if (ve.hasErrors())
		{
			throw ve;
		}
	}

}
