package test.projet.kernel.jsf.manager;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.annotation.PostConstruct;
import javax.enterprise.context.SessionScoped;
import javax.faces.context.FacesContext;
import javax.inject.Named;

@Named
@SessionScoped
public class LocaleManager implements Serializable
{
	public static final long serialVersionUID = 1;
	private List<Locale> locales = new ArrayList<>();
	private Locale locale;

	@PostConstruct
	public void postConstruct()
	{
		Iterator<Locale> it = FacesContext.getCurrentInstance().getApplication().getSupportedLocales();
		while (it.hasNext())
		{
			locales.add(it.next());
		}
	}
	
	public Locale getLocale()
	{
		if (locale == null)
		{
			try
			{
				FacesContext facesContext = FacesContext.getCurrentInstance();
				locale = facesContext.getViewRoot().getLocale();
				if (locale == null)
				{
					locale = facesContext.getExternalContext().getRequestLocale();
				}
			}
			catch (Exception e) {}
			if (locale == null)
			{
				locale = Locale.FRANCE;
			}
		}
		return locale;
	}

	public void setLocale(Locale locale)
	{
		this.locale = locale;
	}

	public List<Locale> getLocales()
	{
		return locales;
	}

	public void setLocales(List<Locale> locales)
	{
		this.locales = locales;
	}

	public void changeLocale(String language)
	{
		changeLocale(language, language.toUpperCase());
	}
	
	public void changeLocale(String language, String country)
	{
		Locale locale = new Locale(language, country);
		setLocale(locale);
	}

	public ResourceBundle getResourceBundle()
	{
		FacesContext facesContext = FacesContext.getCurrentInstance();
		return facesContext.getApplication().getResourceBundle(facesContext, "msg"); 
	}

	public String getLocaleName(String key)
	{
		ResourceBundle bundle = getResourceBundle();
		if (bundle.containsKey(key))
		{
			return bundle.getString(key);
		}
		return key;
	}

}
