package test.projet.personne;

import javax.ejb.EJB;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.faces.view.ViewScoped;
import javax.inject.Named;

import test.projet.ejb.client.dto.DTOPersonne;
import test.projet.ejb.client.dto.DTOPersonneTelephone;
import test.projet.ejb.client.session.MonProjetFacadeLocal;
import test.projet.kernel.AbstractManager;
import test.projet.kernel.enums.ApplicationPagesNamesEnum;

@Named
@ViewScoped
public class PersonneManager extends AbstractManager
{
	private static final long serialVersionUID = 1;
	private DTOPersonne personne;
	private Integer uid;

	@EJB
	private MonProjetFacadeLocal facade;

	public PersonneManager()
	{
		super();
	}

	public Integer getUid()
	{
		return uid;
	}

	public void setUid(Integer uid)
	{
		System.out.println("JSF version " + FacesContext.class.getPackage().getImplementationVersion());
		
		this.uid = uid;
		if (uid != null && (personne == null || !uid.equals(personne.getUid())))
		{
			personne = facade.getPersonne(uid);
		}
	}

	public DTOPersonne getPersonne()
	{
		if (personne == null)
		{
			personne = new DTOPersonne();
		}
		return personne;
	}

	public void setPersonne(DTOPersonne personne)
	{
		this.personne = personne;
	}

	public String display()
	{
		return formatURL(getPageURL(ApplicationPagesNamesEnum.PERSONNE_DETAIL), true);
	}
	
	public String saveOrUpdate()
	{
		try
		{
			facade.saveOrUpdatePersonne(personne);
			sendFacesMessage("Personne enregistrée", FacesMessage.SEVERITY_INFO, true);
			return formatToCaller(true);
		}
		catch (Exception e)
		{
			sendFacesMessage(e, false);			
		}
		return null;
	}
	
	public String delete()
	{
		try
		{
			facade.deletePersonne(personne.getUid());
			sendFacesMessage("Personne supprimée", FacesMessage.SEVERITY_INFO, true);
			return formatToCaller(true);
		}
		catch (Exception e)
		{
			sendFacesMessage("Erreur lors de la suppression de la personne", FacesMessage.SEVERITY_ERROR, false);			
		}
		return null;
	}

	public void addTelephone()
	{
		personne.getTelephones().add(new DTOPersonneTelephone());
	}
	
	public boolean autorisation(String action)
	{
		if (personne == null)
		{
			return false;
		}
		if ("delete".equals(action) && personne.getUid() != null)
		{
			return true;
		}
		if ("save".equals(action))
		{
			return true;
		}
		return false;
	}
}